from __future__ import with_statement

import os, sys
from alembic import context
from sqlalchemy import engine_from_config, pool
from logging.config import fileConfig

# Alembic needs to know where to find the configuration.
additional_path = os.path.dirname(os.path.abspath(__file__)) + "/../../"

if "CONFIG_DIR" not in os.environ:
    local_configdir = os.path.join(additional_path, "etc")
    if os.path.exists(local_configdir):
        os.environ['CONFIG_DIR'] = local_configdir

sys.path = [additional_path] + sys.path

from resallocserver.app import app
from resallocserver.models import Base

# this is the Alembic Config object, which provides
# access to the values within the .ini file in use.
config = context.config

# Interpret the config file for Python logging.
# This line sets up loggers basically.
if (os.path.basename(sys.argv[0])) == 'alembic':
    fileConfig(config.config_file_name)

# add your model's MetaData object here for 'autogenerate' support
# from myapp import mymodel
# target_metadata = mymodel.Base.metadata
target_metadata = Base.metadata

# other values from the config, defined by the needs of env.py,
# can be acquired:
# my_important_option = config.get_main_option("my_important_option")
# ... etc.


def run_migrations_offline():
    """Run migrations in 'offline' mode.

    This configures the context with just a URL
    and not an Engine, though an Engine is acceptable
    here as well.  By skipping the Engine creation
    we don't even need a DBAPI to be available.

    Calls to context.execute() here emit the given string to the
    script output.

    """
    url = app.config['db_url']
    context.configure(
        url=url, target_metadata=target_metadata, literal_binds=True,
        render_as_batch=True,
    )

    with context.begin_transaction():
        context.run_migrations()


def run_migrations_online():
    """Run migrations in 'online' mode.

    In this scenario we need to create an Engine
    and associate a connection with the context.

    """
    connectable = app.engine
    with connectable.connect() as connection:
        context.configure(
            connection=connection,
            target_metadata=target_metadata,
            render_as_batch=True,
        )

        with context.begin_transaction():
            context.run_migrations()

if context.is_offline_mode():
    run_migrations_offline()
else:
    run_migrations_online()
